/*
Copyright (C) 2011 Mark Chandler (Desura Net Pty Ltd)
Copyright (C) 2014 Bad Juju Games, Inc.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.

Contact us at legal@badjuju.com.
*/

#ifndef DESURA_COLOR_H
#define DESURA_COLOR_H

#include <string.h>

class Color
{
public:
	Color(uint8 r, uint8 g, uint8 b, uint8 a = 0)
	: red(r)
	, green(g)
	, blue(b)
	, alpha(a)
	{
	}

	Color(const Color *t)
	: red(0)
	, green(0)
	, blue(0)
	, alpha(255)
	{
		if (t)
		{
			red = t->red;
			green = t->green;
			blue = t->blue;
			alpha = t->alpha;
		}
	}

	Color(const Color &t)
	: red(t.red)
	, green(t.green)
	, blue(t.blue)
	, alpha(t.alpha)
	{
	}

	Color(int val)
	: red((val>>24)&0xFFFFFF)
	, green((val>>16)&0xFFFFFF)
	, blue((val>> 8)&0xFFFFFF)
	, alpha((val>> 0)&0xFFFFFF)
	{
	}

	Color(const char* text)
	: red(0)
	, green(0)
	, blue(0)
	, alpha(255)
	{
		if (!text)
			return;

		uint32 textlen = (uint32) strlen(text);

		if (textlen == 0)
		{
		}
		else if (text[0] == '#')
		{
			uint32 round = 0;
			uint32 count = 1;
			char hex[3];

			while (count < textlen)
			{
				hex[0] = text[count];
				count++;
				hex[1] = text[count];
				count++;
				hex[2]='\0';

				uint8 val = UTIL::MISC::hextoDec(hex);

				switch (round)
				{
					case 0: red = val;
						break;
					case 1: green = val;
						break;
					case 2: blue = val;
						break;
					case 3: alpha = val;
						break;
					default: count = textlen;
						break;
				}
				round++;
			}
		}
		else
		{
			red = Safe::atoi(text);

			char textcpy[255] = {0};
#ifdef WIN32
			strcpy_s(textcpy, 255, text);
#else
			strcpy(textcpy, text);
#endif

			char* g = strstr(textcpy+1, " ");
			if (g)
			{
				green = Safe::atoi(g);

				char* b = strstr(g+1, " ");
				if (b)
				{
					blue = atoi(b);

					char* a = strstr(b+1, " ");
					if (a)
						alpha = Safe::atoi(a);
				}
			}
		}
	}

	Color()
	: red(0)
	, green(0)
	, blue(0)
	, alpha(0)
	{
	}

	uint8 red;
	uint8 green;
	uint8 blue;
	uint8 alpha;

	uint32 getColor(){return (red<<24) + (green<<16) + (blue<<8) + alpha;}

	//wxWidgets color
	operator unsigned long()
	{
		return (blue<<16) + (green<<8) + red;
	}
};


#endif
