/*
Copyright (C) 2011 Mark Chandler (Desura Net Pty Ltd)
Copyright (C) 2014 Bad Juju Games, Inc.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.

Contact us at legal@badjuju.com.
*/

#ifndef DESURA_INTERNALLINK_H
#define DESURA_INTERNALLINK_H
#ifdef _WIN32
#pragma once
#endif

#include "MainApp.h"
#include "ItemForm.h"
#include "LinkArgs.h"

namespace UserCore
{
	namespace Misc
	{
		class NewsItem;
	}
}

class wxWindow;


class InternalLink
{
public:
	InternalLink(wxWindow *parent);
	~InternalLink();

	void handleInternalLink(const char* link);
	void handleInternalLink(DesuraId id, uint8 action, const LinkArgs &argsList = LinkArgs());
	void closeForm(int32 wxId);

	void closeAll();
	void showNews(const std::vector<gcRefPtr<UserCore::Misc::NewsItem>> &newsItems, const std::vector<gcRefPtr<UserCore::Misc::NewsItem>> &giftItems);
	void showAppUpdate(uint32 version);

protected:
	void showPreorderPrompt(DesuraId id, bool isPreload);

	void installedWizard();
	void showSettings(LinkArgs &args);

	void installCheck(DesuraId id);
	void installTestMCF(DesuraId id, LinkArgs args);
	void uploadMCF(DesuraId id);
	void resumeUploadMCF(DesuraId id, LinkArgs args);
	void createMCF(DesuraId id);


	void cleanComplexMod(DesuraId id);
	void installItem(DesuraId id, LinkArgs args);
	void launchItem(DesuraId id, LinkArgs args);
	void verifyItem(DesuraId id, LinkArgs args);
	void uninstallMCF(DesuraId id);

	void updateItem(DesuraId id, LinkArgs args);

	void showEULA(DesuraId id);
	void showUpdateLog(DesuraId id);
	void showUpdateLogApp(uint32 version);
	void showNeedToRun(DesuraId mid, DesuraId pid);

	void showGameDisk(DesuraId id, const char* exe, bool cdKey);
	void showUpdateForm(DesuraId id, LinkArgs args);
	void showCDKey(DesuraId id, LinkArgs args);
	void showExeSelect(DesuraId id, bool hasSeenCDKey = false);

	void setPauseItem(DesuraId id, bool state = true);
	void switchBranch(DesuraId id, LinkArgs args);


	void regForm(DesuraId id, gcFrame *form);
	bool checkForm(DesuraId id);

	void onUploadTrigger(ut& info);

	void showPrompt(DesuraId id, LinkArgs args);

	UI::Forms::ItemForm* showItemForm(DesuraId id, UI::Forms::INSTALL_ACTION action, bool showForm, LinkArgs args = LinkArgs());
	UI::Forms::ItemForm* showItemForm(DesuraId id, UI::Forms::INSTALL_ACTION action, MCFBranch branch = MCFBranch(), MCFBuild build = MCFBuild(), bool showForm = true, LinkArgs args = LinkArgs());

	bool processItemLink(bool &badLink, std::vector<gcString> &list, const char* link);
	bool switchTab(bool &badLink, std::vector<gcString> &list, const char* link);

	bool checkForPreorder(DesuraId id);

private:
	int32 m_iNewsFormId;
	int32 m_iGiftFormId;
	wxWindow* m_pParent;

	std::map<uint64, gcFrame*> m_mFormMap;
	std::vector<wxFrame*> m_vSubForms;

	std::map<gcString, gcFrame*> m_mWaitingItemFromMap;
};


#endif //DESURA_INTERNALLINK_H
