/*
Copyright (C) 2011 Mark Chandler (Desura Net Pty Ltd)
Copyright (C) 2014 Bad Juju Games, Inc.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.

Contact us at legal@badjuju.com.
*/

#ifndef DESURA_MAINAPP_H
#define DESURA_MAINAPP_H

#include "MainAppI.h"


#include "wx/app.h"
#include "webcore/WebCoreI.h"
#include "usercore/UserCoreI.h"
#include "usercore/UploadInfoThreadI.h"
#include "LinkArgs.h"

class gcMessageDialog;

extern "C"
CEXPORT void* CreateMainApp();

extern bool isSafeUrl(const char* url);
extern gcRefPtr<WebCore::WebCoreI> GetWebCore();
extern gcRefPtr<UserCore::UserI> GetUserCore();

const char* GetAppVersion();
const char* GetUserCoreVersion();
const char* GetWebCoreVersion();

class ut
{
public:
	gcString path;
	DesuraId id;
	wxWindow *caller;
};


class LoginForm;
class wxOnAppUpdateEvent;
class wxTaskBarIcon;
class MainForm;

namespace UserCore
{
	namespace Thread
	{
		class UserThreadI;
	}

	namespace Misc
	{
		class NewsItem;
	}
}

namespace UnitTest
{
	class MainAppFixture;
}

class InternalLinkInfo
{
public:
	DesuraId id;
	uint8 action;
	LinkArgs args;
};

class InternalLink;
class TaskBarIcon;
class gcUnitTestForm;

#ifdef WIN32
class ProxyControl
{
public:
	ProxyControl();
	~ProxyControl();

	void ForceProxyOff();
	void RestoreUserProxySettings();

private:
	bool m_UserSettingsNeedRestoring;

	uint32 m_UserProxyEnabled;
	unsigned char m_UserDefaultConnectionSettingsControlByte;
};
#endif

class MainAppNoUI
{
public:
	friend class UnitTest::MainAppFixture;

	void onGiftUpdate(std::vector<gcRefPtr<UserCore::Misc::NewsItem>>& itemList);

	EventV onNotifyGiftUpdateEvent;

	std::mutex m_NewsLock;
	std::vector<gcRefPtr<UserCore::Misc::NewsItem>> m_vGiftItems;
};

class MainApp : public MainAppI, public MainAppProviderI, protected MainAppNoUI
{
public:
	MainApp();
	virtual ~MainApp();

	void run();
	//wx Function override
	void Init(int argc, wxCmdLineArgsArray &argv);
	void showMainWindow(bool raise = false);

	void onTBIMenuSelect(wxCommandEvent& event);

	void onLoginAccepted(bool saveLoginInfo = false, bool autologin = false);

	void logIn(const char* user, const char* pass);
	void logOut(bool bShowLogin = true, bool autoLogin = false, bool webLoggedOut = false);

	void offlineMode();
	void goOffline();

	bool isOffline();
	bool isLoggedIn();

	void handleInternalLink(const char* link) override;
	void handleInternalLink(DesuraId id, uint8 action, const LinkArgs& args) override;

	void closeMainForm();
	void closeForm(int32 id);
#ifdef NIX
	void toggleCurrentForm();
#endif

	void processWildCards(WCSpecialInfo &info, wxWindow* parent);

	gcFrame* getMainWindow();
	wxWindow* getTopLevelWindow();

	virtual void disableQuietMode();
	virtual bool isQuietMode();

	virtual EventV* getLoginEvent();


	void newAccountLogin(const char* username, const char* cookie) override;
	void newAccountLoginError(const char* szErrorMessage) override;

	const char* getProvider() const override;

	void showUnitTest();

protected:
	void showLogin(bool skipAutoLogin = false, bool webLoggedOut = false);
	void showOffline();
	void showMainForm(bool raise = false, bool show = true);

	void createTaskBarIcon();
	void onClose(wxCloseEvent& event);

	void changeAccountState(DesuraId id);

	void loadUrl(const char* url, PAGE page);
	void showProfile(DesuraId id, const LinkArgs &args = LinkArgs());
	void showDevProfile(DesuraId id);
	void showDevPage(DesuraId id);
	void showNews();
	void showPlay();
	void showPage(PAGE page);

	void getSteamUser(WCSpecialInfo *info, wxWindow *parent);

	void onNewsUpdate(std::vector<gcRefPtr<UserCore::Misc::NewsItem>>& itemList);

	void onNeedCvar(UserCore::Misc::CVar_s& info);

	void onAppUpdateProg(uint32& prog);
	void onAppUpdate(UserCore::Misc::UpdateInfo& info);

	void onCookieUpdate();
	void onLoggedOut();

	void onLoginAcceptedCB(std::pair<bool,bool> &loginInfo);

	void onPipeDisconnect();

	void onInternalStrLink(gcString &link);
	void onInternalLink(InternalLinkInfo& info);

	void onNotifyGiftUpdate();

	void showConsole();

	void setProvider(const char* szProvider) override;

	void userLoggedOut() override;

	Event<std::pair<bool,bool>> onLoginAcceptedEvent;
	EventV onLoginEvent;

	Event<InternalLinkInfo> onInternalLinkEvent;
	Event<gcString> onInternalLinkStrEvent;
	EventV onLoggedOutEvent;

private:
	friend class Desura;
	gcMessageDialog *m_pOfflineDialog = nullptr;

	std::vector<gcRefPtr<UserCore::Misc::NewsItem>> m_vNewsItems;
	gcRefPtr<UserCore::Thread::UserThreadI> m_pDumpThread;

	bool m_bQuiteMode = false;
	bool m_bLoggedIn = false;
	APP_MODE m_iMode = APP_MODE::MODE_LOGOUT;

	gcString m_strServiceProvider;
	gcString m_szDesuraCache;

#ifdef WITH_GTEST
	gcUnitTestForm* m_UnitTestForm = nullptr;
#endif

	LoginForm* m_wxLoginForm = nullptr;
	MainForm* m_wxMainForm = nullptr;
	TaskBarIcon* m_wxTBIcon = nullptr;

	InternalLink *m_pInternalLink = nullptr;

	std::mutex m_UserLock;

#ifdef WIN32
	ProxyControl m_ProxyControl;
#endif

	friend void cc_NewsTest_cc_func(std::vector<gcString> &argv);
	friend void cc_GiftTest_cc_func(std::vector<gcString> &argv);

#ifdef DEBUG
	friend class PopUpThread;
#endif

	friend void cc_test_news_cc_func(std::vector<gcString> &vArgList);
};


extern MainAppI* g_pMainApp;

#endif
